var _extends = Object.assign || function (target) { for (var i = 1; i < arguments.length; i++) { var source = arguments[i]; for (var key in source) { if (Object.prototype.hasOwnProperty.call(source, key)) { target[key] = source[key]; } } } return target; };

function _classCallCheck(instance, Constructor) { if (!(instance instanceof Constructor)) { throw new TypeError("Cannot call a class as a function"); } }

function _possibleConstructorReturn(self, call) { if (!self) { throw new ReferenceError("this hasn't been initialised - super() hasn't been called"); } return call && (typeof call === "object" || typeof call === "function") ? call : self; }

function _inherits(subClass, superClass) { if (typeof superClass !== "function" && superClass !== null) { throw new TypeError("Super expression must either be null or a function, not " + typeof superClass); } subClass.prototype = Object.create(superClass && superClass.prototype, { constructor: { value: subClass, enumerable: false, writable: true, configurable: true } }); if (superClass) Object.setPrototypeOf ? Object.setPrototypeOf(subClass, superClass) : subClass.__proto__ = superClass; }

import Persistence from "../Persistence";

var f_collection = Symbol("Field firestore collection");
var f_db = Symbol("Field firestore db");

var m_extractId = Symbol("Method extract Id");
var m_getRef = Symbol("Method get firestore ref");

var hidden_snapshot = Symbol("Hidden link to snapshot");

export var getSnapshot = function getSnapshot(data) {
    return data[hidden_snapshot];
};

var wrapSnapshot = function wrapSnapshot(snapshot) {
    var _extends2;

    return _extends({}, snapshot.data(), (_extends2 = {}, _extends2[hidden_snapshot] = snapshot, _extends2));
};

var extractDataFromSnapshot = function extractDataFromSnapshot(snapshot) {
    if (snapshot.docs) {
        return snapshot.docs.map(wrapSnapshot);
    } else {
        if (!snapshot.exists) return null;
        return wrapSnapshot(snapshot);
    }
};

//FIXME: Use a custom ref and add a ref builder

var FirestorePersistence = function (_Persistence) {
    _inherits(FirestorePersistence, _Persistence);

    function FirestorePersistence(db) {
        var _ref = arguments.length > 1 && arguments[1] !== undefined ? arguments[1] : {},
            mapDocId = _ref.mapDocId;

        _classCallCheck(this, FirestorePersistence);

        var _this = _possibleConstructorReturn(this, _Persistence.call(this));

        _this[f_db] = db;
        if (mapDocId && typeof mapDocId !== "function") throw new Error("Option mapDocId is expected to be a function value => id");
        _this[m_extractId] = mapDocId;
        _this[f_collection] = null;
        return _this;
    }

    FirestorePersistence.prototype.at = function at(context) {
        var clone = new FirestorePersistence(this[f_db], { mapDocId: this[m_extractId] });
        clone[f_collection] = this[f_db].collection(context);
        return clone;
    };

    FirestorePersistence.prototype[m_getRef] = function (value) {
        if (this[f_collection] === null) throw new Error("Firestore persistence don't allow access to root context. Use at(context) before.");
        if (this[m_extractId]) {
            return this[f_collection].doc(this[m_extractId](value));
        } else {
            return this[f_collection].doc();
        }
    };

    FirestorePersistence.prototype.save = function save(value, at) {
        var ref = at || this[m_getRef](value);
        return ref.set(value).then(function () {
            return ref;
        });
    };

    FirestorePersistence.prototype.read = function read(ref) {
        return ref.get().then(extractDataFromSnapshot);
    };

    FirestorePersistence.prototype.watch = function watch(ref, onChange, onError) {
        return ref.onSnapshot(function (snapshot) {
            return onChange(extractDataFromSnapshot(snapshot));
        }, onError);
    };

    FirestorePersistence.prototype.delete = function _delete(ref) {
        //FIXME: Handle collections
        return ref.delete();
    };

    return FirestorePersistence;
}(Persistence);

export default FirestorePersistence;