var _extends = Object.assign || function (target) { for (var i = 1; i < arguments.length; i++) { var source = arguments[i]; for (var key in source) { if (Object.prototype.hasOwnProperty.call(source, key)) { target[key] = source[key]; } } } return target; };

function _classCallCheck(instance, Constructor) { if (!(instance instanceof Constructor)) { throw new TypeError("Cannot call a class as a function"); } }

/**
 * @export
 * @class EntitySchema
 */
export var EntitySchema =
/**
 * Creates an instance of EntitySchema.
 * @param {Object.<string, import("./types/Type.js").Type>} typesMap 
 * @param {any} validations 
 * @memberof EntitySchema
 */
function EntitySchema(typesMap, validations) {
    _classCallCheck(this, EntitySchema);

    this.typesMap = typesMap;
    this.validations = validations;
};

var m_checkInstructions = Symbol("Method checkInstructions");
var m_apply = Symbol("Method apply");
var f_instructions = Symbol("Field instructions");

var EntitySchemaBuilder = function () {
    function EntitySchemaBuilder() {
        _classCallCheck(this, EntitySchemaBuilder);

        this[f_instructions] = {
            typesMap: {}
        };
    }

    /**
     * 
     * 
     * @param {any} attribute 
     * @returns {EntitySchemaBuilder}
     * @memberof EntitySchemaBuilder
     */


    EntitySchemaBuilder.prototype.addAttribute = function addAttribute(name, type) {

        if (this[f_instructions].typesMap[name] !== undefined) throw new Error("There is already an attribute with this name");
        return this[m_apply](function (builder) {
            builder[f_instructions].typesMap[name] = type;
        });
    };

    /**
     * @param {Function} change 
     * @returns {EntitySchemaBuilder}
     * @memberof EntitySchemaBuilder
     */


    EntitySchemaBuilder.prototype[m_apply] = function (change) {
        //For immutability
        var newBuilder = new EntitySchemaBuilder();
        newBuilder[f_instructions].typesMap = _extends({}, this[f_instructions].typesMap);
        change(newBuilder);
        return newBuilder;
    };

    EntitySchemaBuilder.prototype[m_checkInstructions] = function () {};

    /**
     * 
     * 
     * @returns {EntitySchema}
     * @memberof EntitySchemaBuilder
     */


    EntitySchemaBuilder.prototype.build = function build() {
        this[m_checkInstructions]();
        return new EntitySchema(this[f_instructions].typesMap);
    };

    return EntitySchemaBuilder;
}();

export { EntitySchemaBuilder as default };