import { Type } from "./Type";
import { makePredicate, isEmpty } from "./predicates";
import ValidationError from "./ValidationError";

var isArray = makePredicate(Array.isArray);

/**
 * @typedef ArrayType
 * @property {(...types : Type[]) => ArrayType} of
 */

/**
 * @type {Type & ArrayType}
 */
var array = new Type().extendWithValidators(function (value) {
  return isEmpty.or(isArray)(value) ? [] : [new ValidationError("numbani:validations.array.invalidType", { value: value })];
});

/**
 * 
 * @param {Array.<Type>} types
 */
array.of = function () {
  for (var _len = arguments.length, types = Array(_len), _key = 0; _key < _len; _key++) {
    types[_key] = arguments[_key];
  }

  var newType = this.extendWithValidators(function (value) {
    return isEmpty.or(isArray.and(function (value) {
      return value.every(function (element) {
        return types.some(function (type) {
          return type.accepts(element);
        });
      });
    }))(value) ? [] : [new ValidationError("numbani:validations.array.invalidElementType", { value: value, types: types })];
  });
  newType.addInfo("of", types);
  return newType;
};

array.max = function (length) {
  var newType = this.extendWithValidators(function (value) {
    return isEmpty.or(isArray.and(function (arr) {
      return arr.length <= length;
    }))(value) ? [] : [new ValidationError("numbani:validations.array.invalidMaxLength", { value: value, expected: length, actual: value.length })];
  });
  newType.addInfo("array.max", Math.min(newType.getInfo("array.max") || Number.POSITIVE_INFINITY, length));
  return newType;
};

array.min = function (length) {
  var newType = this.extendWithValidators(function (value) {
    return isEmpty.or(isArray.and(function (arr) {
      return arr.length >= length;
    }))(value) ? [] : [new ValidationError("numbani:validations.array.invalidMinLength", { value: value, expected: length, actual: value.length })];
  });
  newType.addInfo("array.min", Math.max(newType.getInfo("array.min") || Number.NEGATIVE_INFINITY, length));
  return newType;
};

export default array;