import { Type } from "./Type";
import { makePredicate, isEmpty } from "./predicates";
import ValidationError from "./ValidationError";

var isNumber = makePredicate(function (value) {
  return typeof value === 'number' && isFinite(value);
});

var number = new Type().extendWithValidators(function (value) {
  return isEmpty.or(isNumber)(value) ? [] : [new ValidationError("numbani:validations.number.invalidType", { value: value })];
});

number.min = function (minValue) {
  var newType = this.extendWithValidators(function (value) {
    return isEmpty.or(isNumber.and(function (n) {
      return n >= minValue;
    }))(value) ? [] : [new ValidationError("numbani:validations.number.invalidMinValue", { value: value, expected: minValue, actual: value })];
  });
  newType.addInfo("number.min", Math.max(newType.getInfo("number.min") || Number.NEGATIVE_INFINITY, minValue));
  return newType;
};

number.max = function (maxValue) {
  var newType = this.extendWithValidators(function (value) {
    return isEmpty.or(isNumber.and(function (n) {
      return n <= maxValue;
    }))(value) ? [] : [new ValidationError("numbani:validations.number.invalidMaxValue", { value: value, expected: maxValue, actual: value })];
  });
  newType.addInfo("number.max", Math.min(newType.getInfo("number.max") || Number.POSITIVE_INFINITY, maxValue));
  return newType;
};

export default number;