import { Type } from "./Type";
import { makePredicate, isEmpty } from './predicates';
import ValidationError from "./ValidationError";

var isString = makePredicate(function (value) {
    return typeof value === "string";
});

var string = new Type().extendWithValidators(function (value) {
    return isEmpty.or(isString)(value) ? [] : [new ValidationError("numbani:validations.string.invalidType", { value: value })];
});

string.max = function (length) {
    var newType = this.extendWithValidators(function (value) {
        return isEmpty.or(isString.and(function (str) {
            return str.length <= length;
        }))(value) ? [] : [new ValidationError("numbani:validations.string.invalidMaxLength", { value: value, expected: length, actual: value.length })];
    });
    newType.addInfo("string.max", Math.min(newType.getInfo("string.max") || Number.POSITIVE_INFINITY, length));
    return newType;
};

string.min = function (length) {
    var newType = this.extendWithValidators(function (value) {
        return isEmpty.or(isString.and(function (str) {
            return str.length >= length;
        }))(value) ? [] : [new ValidationError("numbani:validations.string.invalidMinLength", { value: value, expected: length, actual: value.length })];
    });
    newType.addInfo("string.min", Math.max(newType.getInfo("string.min") || Number.NEGATIVE_INFINITY, length));
    return newType;
};
/**
 * 
 * @param {RegExp} regex 
 */
string.pattern = function (regex) {
    var newType = this.extendWithValidators(function (value) {
        return isEmpty.or(isString.and(function (str) {
            return regex.test(str);
        }))(value) ? [] : [new ValidationError("numbani:validations.string.patternNotMatched", { value: value, expected: regex, actual: value })];
    });
    newType.addInfo("pattern", (newType.getInfo("pattern") || []).push(regex));
    return newType;
};
export default string;