import React from 'react';
import { mount } from 'enzyme';
import expect from 'expect';

import EntityManagerBuilder from '../model/EntityManagerBuilder';
import Persistence from '../model/persistence/provider/InMemory';
import EntitySchemaBuilder from '../model/schema/EntitySchemaBuilder';
import types from '../model/schema/types';
import bootstrap4UiLib from './lib/bootstrap4';
import UI from '.';

var entityManager = new EntityManagerBuilder().withPersistenceProvider(new Persistence({ initialDB: { cars: { mine: { brand: "Toyota" } } } })).build();

var carSchema = new EntitySchemaBuilder().addAttribute("brand", types.String.required().min(1).max(255))
//  .addAttribute("color", types.Enum.required().oneOf("red", "blue","green","black"))
.build();

var carsRepo = entityManager.register("cars", { schema: carSchema }); // <<<<< Here we have our EntityRepo

export var CarView = UI(bootstrap4UiLib).forRepo(carsRepo); //<<<<< And then we build the EntityView

describe("EntityView.one", function () {

    it('Allow simple attribute interpolation', function () {
        var wrapper = mount(React.createElement(
            CarView.one,
            { value: { brand: 'Toyota' } },
            React.createElement(CarView.fields.brand, null)
        ));

        expect(wrapper.text()).toBe('Toyota');
    });
});