"use strict";

exports.__esModule = true;
exports.default = exports.EntityManager = undefined;

var _Persistence = require("./persistence/Persistence");

var _Persistence2 = _interopRequireDefault(_Persistence);

var _EntitySchemaBuilder = require("./schema/EntitySchemaBuilder");

var _EntityRepo = require("./core/EntityRepo");

var _EntityRepo2 = _interopRequireDefault(_EntityRepo);

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

function _classCallCheck(instance, Constructor) { if (!(instance instanceof Constructor)) { throw new TypeError("Cannot call a class as a function"); } }

var m_getEntityRepo = Symbol("Method getEntityRepo");
var m_setEntityRepo = Symbol("Method setEntityRepo");
var f_EM_private = Symbol("Field private for Entity Manager");

var EntityManager = exports.EntityManager = function () {
    function EntityManager(persistenceProvider) {
        _classCallCheck(this, EntityManager);

        this[f_EM_private] = {
            persistenceProvider: persistenceProvider,
            entitiesReposMap: {}
        };
    }

    /**
     * Build an entity that will be managed by this manager;
     * Returns an entity repository object, to interact with this entity.
     * @param {string} name - Unique name of this entity in the mananger
     * @param {Object} options
     * @param {EntitySchema} options.schema 
     * @returns {EntityRepo}
     * @memberof EntityManager
     */


    EntityManager.prototype.register = function register(name, _ref) {
        var schema = _ref.schema,
            behaviours = _ref.behaviours;

        if (!(schema instanceof _EntitySchemaBuilder.EntitySchema)) throw new Error("entitySchema isn't an instance of class numbani/model/EntitySchema");

        if (this[m_getEntityRepo](name) !== undefined) throw new Error("An entity have already been registered with this name");

        this[m_setEntityRepo](name, new _EntityRepo2.default(schema, this[f_EM_private].persistenceProvider.at(name)));

        return this[m_getEntityRepo](name);
    };

    EntityManager.prototype[m_setEntityRepo] = function (name, repo) {
        this[f_EM_private].entitiesReposMap[name] = repo;
    };

    EntityManager.prototype[m_getEntityRepo] = function (name) {
        return this[f_EM_private].entitiesReposMap[name];
    };

    return EntityManager;
}();

var m_checkInstructions = Symbol("Method checkInstructions");
var m_apply = Symbol("Method apply");
var f_instructions = Symbol("Field instructions");

var EntityManagerBuilder = function () {
    function EntityManagerBuilder() {
        _classCallCheck(this, EntityManagerBuilder);

        this[f_instructions] = {
            persistenceProvider: null
        };
    }

    /**
     * 
     * 
     * @param {Persitence} provider 
     * @returns {EntityManagerBuilder}
     * @memberof EntityManagerBuilder
     */


    EntityManagerBuilder.prototype.withPersistenceProvider = function withPersistenceProvider(provider) {

        if (!(provider instanceof _Persistence2.default)) throw new Error("Provider isn't an instance of class numbani/model/persistence/Persistence");

        return this[m_apply](function (builder) {
            builder[f_instructions].persistenceProvider = provider;
        });
    };

    /**
     * 
     * 
     * @param {Function} change 
     * @returns {EntityManagerBuilder}
     * @memberof EntityManagerBuilder
     */


    EntityManagerBuilder.prototype[m_apply] = function (change) {
        //For immutability
        var newBuilder = new EntityManagerBuilder();
        newBuilder[f_instructions].persistenceProvider = this[f_instructions].persistenceProvider;
        change(newBuilder);
        return newBuilder;
    };

    EntityManagerBuilder.prototype[m_checkInstructions] = function () {
        if (this[f_instructions].persistenceProvider === null) throw new Error("Persitence provider is required");
    };

    EntityManagerBuilder.prototype.build = function build() {
        this[m_checkInstructions]();
        return new EntityManager(this[f_instructions].persistenceProvider);
    };

    return EntityManagerBuilder;
}();

exports.default = EntityManagerBuilder;