'use strict';

exports.__esModule = true;

var _regenerator = require('babel-runtime/regenerator');

var _regenerator2 = _interopRequireDefault(_regenerator);

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

function _asyncToGenerator(fn) { return function () { var gen = fn.apply(this, arguments); return new Promise(function (resolve, reject) { function step(key, arg) { try { var info = gen[key](arg); var value = info.value; } catch (error) { reject(error); return; } if (info.done) { resolve(value); } else { return Promise.resolve(value).then(function (value) { step("next", value); }, function (err) { step("throw", err); }); } } return step("next"); }); }; }

function _classCallCheck(instance, Constructor) { if (!(instance instanceof Constructor)) { throw new TypeError("Cannot call a class as a function"); } }

/**
 * @export
 * @class EntityRepo
 */
var EntityRepo = function () {
    /**
     * Creates an instance of EntityRepo.
     * @param {import("../schema/EntitySchemaBuilder.js").EntitySchema} entitySchema 
     * @param {import("../persistence/Persistence.js").default} persistence 
     * @memberof EntityRepo
     */
    function EntityRepo(entitySchema, persistence) {
        _classCallCheck(this, EntityRepo);

        this.persistence = persistence;
        this.schema = entitySchema;
    }

    EntityRepo.prototype.validate = function () {
        var _ref = _asyncToGenerator( /*#__PURE__*/_regenerator2.default.mark(function _callee(value) {
            var validationErrors, attribute;
            return _regenerator2.default.wrap(function _callee$(_context) {
                while (1) {
                    switch (_context.prev = _context.next) {
                        case 0:
                            // Perform attributes validation
                            /** @type {Object.<string, Array.<import("../schema/types/ValidationError").default>} */
                            validationErrors = {};
                            // --Type validation

                            for (attribute in this.schema.typesMap) {
                                validationErrors[attribute] = [].concat(this.schema.typesMap[attribute].getValidationErrors(value[attribute]));
                            }
                            //  TODO: Async validation
                            //-- TODO: Cross field validation
                            return _context.abrupt('return', validationErrors);

                        case 3:
                        case 'end':
                            return _context.stop();
                    }
                }
            }, _callee, this);
        }));

        function validate(_x) {
            return _ref.apply(this, arguments);
        }

        return validate;
    }();

    EntityRepo.prototype.save = function () {
        var _ref2 = _asyncToGenerator( /*#__PURE__*/_regenerator2.default.mark(function _callee2(value, at) {
            var validationErrors, validationErrorsCount, attribute, error, final, _attribute;

            return _regenerator2.default.wrap(function _callee2$(_context2) {
                while (1) {
                    switch (_context2.prev = _context2.next) {
                        case 0:
                            _context2.next = 2;
                            return this.validate(value);

                        case 2:
                            validationErrors = _context2.sent;
                            validationErrorsCount = 0;

                            for (attribute in validationErrors) {
                                validationErrorsCount += validationErrors[attribute].length;
                            }

                            if (!(validationErrorsCount > 0)) {
                                _context2.next = 10;
                                break;
                            }

                            //TODO: Emit on-validation-failure
                            error = new Error('Invalid value');

                            error.validationErrors = validationErrors;
                            error.validationErrorsCount = validationErrorsCount;
                            return _context2.abrupt('return', Promise.reject(error));

                        case 10:
                            //TODO: Emit pre-save&post-validation event

                            final = {};

                            for (_attribute in this.schema.typesMap) {
                                final[_attribute] = value[_attribute];
                            }
                            // Persist in database and return reference
                            return _context2.abrupt('return', this.persistence.save(final, at).then(function (ref) {
                                ////TODO: Emit post-save
                                return ref;
                            }));

                        case 13:
                        case 'end':
                            return _context2.stop();
                    }
                }
            }, _callee2, this);
        }));

        function save(_x2, _x3) {
            return _ref2.apply(this, arguments);
        }

        return save;
    }();

    EntityRepo.prototype.watch = function watch(ref, onChange, onError) {
        return this.persistence.watch(ref, onChange, onError);
        //TODO: Emit post read
    };

    EntityRepo.prototype.read = function read(ref) {
        return this.persistence.read(ref);
        //TODO: Emit post read
    };

    EntityRepo.prototype.delete = function _delete(ref) {
        //TODO: Emit pre and post delete event
        return this.persistence.delete(ref);
    };

    return EntityRepo;
}();

exports.default = EntityRepo;
module.exports = exports['default'];