"use strict";

exports.__esModule = true;

var _Type = require("./Type");

var _predicates = require("./predicates");

var _ValidationError = require("./ValidationError");

var _ValidationError2 = _interopRequireDefault(_ValidationError);

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

var isArray = (0, _predicates.makePredicate)(Array.isArray);

/**
 * @typedef ArrayType
 * @property {(...types : Type[]) => ArrayType} of
 */

/**
 * @type {Type & ArrayType}
 */
var array = new _Type.Type().extendWithValidators(function (value) {
  return _predicates.isEmpty.or(isArray)(value) ? [] : [new _ValidationError2.default("numbani:validations.array.invalidType", { value: value })];
});

/**
 * 
 * @param {Array.<Type>} types
 */
array.of = function () {
  for (var _len = arguments.length, types = Array(_len), _key = 0; _key < _len; _key++) {
    types[_key] = arguments[_key];
  }

  var newType = this.extendWithValidators(function (value) {
    return _predicates.isEmpty.or(isArray.and(function (value) {
      return value.every(function (element) {
        return types.some(function (type) {
          return type.accepts(element);
        });
      });
    }))(value) ? [] : [new _ValidationError2.default("numbani:validations.array.invalidElementType", { value: value, types: types })];
  });
  newType.addInfo("of", types);
  return newType;
};

array.max = function (length) {
  var newType = this.extendWithValidators(function (value) {
    return _predicates.isEmpty.or(isArray.and(function (arr) {
      return arr.length <= length;
    }))(value) ? [] : [new _ValidationError2.default("numbani:validations.array.invalidMaxLength", { value: value, expected: length, actual: value.length })];
  });
  newType.addInfo("array.max", Math.min(newType.getInfo("array.max") || Number.POSITIVE_INFINITY, length));
  return newType;
};

array.min = function (length) {
  var newType = this.extendWithValidators(function (value) {
    return _predicates.isEmpty.or(isArray.and(function (arr) {
      return arr.length >= length;
    }))(value) ? [] : [new _ValidationError2.default("numbani:validations.array.invalidMinLength", { value: value, expected: length, actual: value.length })];
  });
  newType.addInfo("array.min", Math.max(newType.getInfo("array.min") || Number.NEGATIVE_INFINITY, length));
  return newType;
};

exports.default = array;
module.exports = exports["default"];