"use strict";

exports.__esModule = true;

var _Type = require("./Type");

var _predicates = require("./predicates");

var _ValidationError = require("./ValidationError");

var _ValidationError2 = _interopRequireDefault(_ValidationError);

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

var isNumber = (0, _predicates.makePredicate)(function (value) {
  return typeof value === 'number' && isFinite(value);
});

var number = new _Type.Type().extendWithValidators(function (value) {
  return _predicates.isEmpty.or(isNumber)(value) ? [] : [new _ValidationError2.default("numbani:validations.number.invalidType", { value: value })];
});

number.min = function (minValue) {
  var newType = this.extendWithValidators(function (value) {
    return _predicates.isEmpty.or(isNumber.and(function (n) {
      return n >= minValue;
    }))(value) ? [] : [new _ValidationError2.default("numbani:validations.number.invalidMinValue", { value: value, expected: minValue, actual: value })];
  });
  newType.addInfo("number.min", Math.max(newType.getInfo("number.min") || Number.NEGATIVE_INFINITY, minValue));
  return newType;
};

number.max = function (maxValue) {
  var newType = this.extendWithValidators(function (value) {
    return _predicates.isEmpty.or(isNumber.and(function (n) {
      return n <= maxValue;
    }))(value) ? [] : [new _ValidationError2.default("numbani:validations.number.invalidMaxValue", { value: value, expected: maxValue, actual: value })];
  });
  newType.addInfo("number.max", Math.min(newType.getInfo("number.max") || Number.POSITIVE_INFINITY, maxValue));
  return newType;
};

exports.default = number;
module.exports = exports["default"];