"use strict";

exports.__esModule = true;

var _Type = require("./Type");

var _predicates = require("./predicates");

var _ValidationError = require("./ValidationError");

var _ValidationError2 = _interopRequireDefault(_ValidationError);

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

var isString = (0, _predicates.makePredicate)(function (value) {
    return typeof value === "string";
});

var string = new _Type.Type().extendWithValidators(function (value) {
    return _predicates.isEmpty.or(isString)(value) ? [] : [new _ValidationError2.default("numbani:validations.string.invalidType", { value: value })];
});

string.max = function (length) {
    var newType = this.extendWithValidators(function (value) {
        return _predicates.isEmpty.or(isString.and(function (str) {
            return str.length <= length;
        }))(value) ? [] : [new _ValidationError2.default("numbani:validations.string.invalidMaxLength", { value: value, expected: length, actual: value.length })];
    });
    newType.addInfo("string.max", Math.min(newType.getInfo("string.max") || Number.POSITIVE_INFINITY, length));
    return newType;
};

string.min = function (length) {
    var newType = this.extendWithValidators(function (value) {
        return _predicates.isEmpty.or(isString.and(function (str) {
            return str.length >= length;
        }))(value) ? [] : [new _ValidationError2.default("numbani:validations.string.invalidMinLength", { value: value, expected: length, actual: value.length })];
    });
    newType.addInfo("string.min", Math.max(newType.getInfo("string.min") || Number.NEGATIVE_INFINITY, length));
    return newType;
};
/**
 * 
 * @param {RegExp} regex 
 */
string.pattern = function (regex) {
    var newType = this.extendWithValidators(function (value) {
        return _predicates.isEmpty.or(isString.and(function (str) {
            return regex.test(str);
        }))(value) ? [] : [new _ValidationError2.default("numbani:validations.string.patternNotMatched", { value: value, expected: regex, actual: value })];
    });
    newType.addInfo("pattern", (newType.getInfo("pattern") || []).push(regex));
    return newType;
};
exports.default = string;
module.exports = exports["default"];