'use strict';

exports.__esModule = true;
exports.AuthServiceConsumer = exports.AuthServiceProvider = undefined;

var _react = require('react');

var _react2 = _interopRequireDefault(_react);

var _propTypes = require('prop-types');

var _propTypes2 = _interopRequireDefault(_propTypes);

var _AuthService = require('../../auth/AuthService');

var _AuthService2 = _interopRequireDefault(_AuthService);

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

function _classCallCheck(instance, Constructor) { if (!(instance instanceof Constructor)) { throw new TypeError("Cannot call a class as a function"); } }

function _possibleConstructorReturn(self, call) { if (!self) { throw new ReferenceError("this hasn't been initialised - super() hasn't been called"); } return call && (typeof call === "object" || typeof call === "function") ? call : self; }

function _inherits(subClass, superClass) { if (typeof superClass !== "function" && superClass !== null) { throw new TypeError("Super expression must either be null or a function, not " + typeof superClass); } subClass.prototype = Object.create(superClass && superClass.prototype, { constructor: { value: subClass, enumerable: false, writable: true, configurable: true } }); if (superClass) Object.setPrototypeOf ? Object.setPrototypeOf(subClass, superClass) : subClass.__proto__ = superClass; }

/**
 * This context allow to provide or consume an AuthService.
 */
var AuthContext = _react2.default.createContext(null);

/**
 * This react component is a shortcut to provide authService to React component.
 * It is useful cause it handles the component lifecycle and update only when needed.
 * Under the hood it is just a react context provider.
 * 
 * @export
 * @class AuthServiceProvider
 * @extends {Component}
 */

var AuthServiceProvider = exports.AuthServiceProvider = function (_Component) {
    _inherits(AuthServiceProvider, _Component);

    function AuthServiceProvider() {
        var _temp, _this, _ret;

        _classCallCheck(this, AuthServiceProvider);

        for (var _len = arguments.length, args = Array(_len), _key = 0; _key < _len; _key++) {
            args[_key] = arguments[_key];
        }

        return _ret = (_temp = (_this = _possibleConstructorReturn(this, _Component.call.apply(_Component, [this].concat(args))), _this), _this.state = {}, _temp), _possibleConstructorReturn(_this, _ret);
    }

    AuthServiceProvider.cloneAuthService = function cloneAuthService(authService) {
        // Force consumer to render, see https://reactjs.org/docs/context.html#caveats
        return Object.assign(Object.create(authService), authService);
    };

    AuthServiceProvider.getDerivedStateFromProps = function getDerivedStateFromProps(props, state) {
        return { authService: AuthServiceProvider.cloneAuthService(props.authService) };
    };

    AuthServiceProvider.prototype.componentDidMount = function componentDidMount() {
        var _this2 = this;

        this.unsubscribe = this.props.authService.onAuthStateChanged(function () {
            _this2.setState({ authService: AuthServiceProvider.cloneAuthService(_this2.props.authService) });
        });
    };

    AuthServiceProvider.prototype.componentWillUnmount = function componentWillUnmount() {
        this.unsubscribe();
    };

    AuthServiceProvider.prototype.render = function render() {
        return _react2.default.createElement(
            AuthContext.Provider,
            { value: this.state.authService },
            this.props.children
        );
    };

    return AuthServiceProvider;
}(_react.Component);

AuthServiceProvider.propTypes = {
    /**
     * The authService you want to provide to React component below
     */
    authService: _propTypes2.default.instanceOf(_AuthService2.default).isRequired

    /**
     * This component allow to consume AuthService. Under the hood it is just a react context consumer.
     */
};var AuthServiceConsumer = exports.AuthServiceConsumer = AuthContext.Consumer;