'use strict';

exports.__esModule = true;

var _react = require('react');

var _react2 = _interopRequireDefault(_react);

var _Entity = require('../model/Entity');

var _propTypes = require('prop-types');

var _propTypes2 = _interopRequireDefault(_propTypes);

var _types = require('../../model/schema/types');

var _types2 = _interopRequireDefault(_types);

var _reactstrap = require('reactstrap');

var _reactNumericInput = require('react-numeric-input');

var _reactNumericInput2 = _interopRequireDefault(_reactNumericInput);

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

function _classCallCheck(instance, Constructor) { if (!(instance instanceof Constructor)) { throw new TypeError("Cannot call a class as a function"); } }

function _possibleConstructorReturn(self, call) { if (!self) { throw new ReferenceError("this hasn't been initialised - super() hasn't been called"); } return call && (typeof call === "object" || typeof call === "function") ? call : self; }

function _inherits(subClass, superClass) { if (typeof superClass !== "function" && superClass !== null) { throw new TypeError("Super expression must either be null or a function, not " + typeof superClass); } subClass.prototype = Object.create(superClass && superClass.prototype, { constructor: { value: subClass, enumerable: false, writable: true, configurable: true } }); if (superClass) Object.setPrototypeOf ? Object.setPrototypeOf(subClass, superClass) : subClass.__proto__ = superClass; }

function renderValidationErrors(props) {
    return props.feedback ? typeof props.feedback === 'function' ? props.feedback(props.validationErrors).map(function (message) {
        return _react2.default.createElement(
            _reactstrap.FormFeedback,
            { style: { display: "block" }, key: message },
            message
        );
    }) : _react2.default.createElement(
        _reactstrap.FormFeedback,
        { style: { display: "block" } },
        props.feedback
    ) : props.validationErrors.map(function (error) {
        return _react2.default.createElement(
            _reactstrap.FormFeedback,
            { style: { display: "block" }, key: JSON.stringify(error) },
            error.display()
        );
    });
}

/**
 * @param {Type} type
 * @returns
 */
function getControlString(type) {
    return function (_Component) {
        _inherits(ControlString, _Component);

        function ControlString(props) {
            _classCallCheck(this, ControlString);

            var _this = _possibleConstructorReturn(this, _Component.call(this, props));

            _this.handleChange = function (event) {
                _this.setState({ value: event.target.value, dirty: true });
                _this.props.onChange(event.target.value);
            };

            _this.handleFocus = function (event) {
                _this.setState({ focused: true });
            };

            _this.handleBlur = function (event) {
                _this.setState({ focused: false, dirty: true });
            };

            _this.state = { dirty: false, focused: false };
            return _this;
        }

        ControlString.getDerivedStateFromProps = function getDerivedStateFromProps(props, state) {
            if (!state.focused) {
                return { value: props.value || "" };
            } else return null;
        };

        ControlString.prototype.render = function render() {
            var valueIsInvalid = this.state.dirty && this.props.validationErrors && this.props.validationErrors.length > 0;
            return _react2.default.createElement(
                _react2.default.Fragment,
                null,
                this.props.edit ? _react2.default.createElement(
                    _react2.default.Fragment,
                    null,
                    _react2.default.createElement(
                        _reactstrap.FormGroup,
                        null,
                        this.props.label && _react2.default.createElement(
                            _reactstrap.Label,
                            null,
                            this.props.label
                        ),
                        _react2.default.createElement(_reactstrap.Input, {
                            type: 'text',
                            value: this.state.value,
                            placeholder: this.props.placeholder,
                            onChange: this.handleChange,
                            onFocus: this.handleFocus,
                            onBlur: this.handleBlur,
                            invalid: valueIsInvalid
                        }),
                        valueIsInvalid && renderValidationErrors(this.props)
                    )
                ) : _react2.default.createElement(
                    _react2.default.Fragment,
                    null,
                    this.props.label && _react2.default.createElement(
                        _reactstrap.Label,
                        null,
                        this.props.label
                    ),
                    ' ',
                    this.props.value || ""
                )
            );
        };

        return ControlString;
    }(_react.Component);
}

/**
 * @param {Type} type
 * @returns
 */
function getControlBoolean(type) {
    return function (_Component2) {
        _inherits(ControlString, _Component2);

        function ControlString(props) {
            _classCallCheck(this, ControlString);

            var _this2 = _possibleConstructorReturn(this, _Component2.call(this, props));

            _this2.handleChange = function (event) {
                _this2.setState({ value: event.target.checked, dirty: true });
                _this2.props.onChange(event.target.checked);
            };

            _this2.handleFocus = function (event) {
                _this2.setState({ focused: true });
            };

            _this2.handleBlur = function (event) {
                _this2.setState({ focused: false, dirty: true });
            };

            _this2.state = { dirty: false, focused: false };
            return _this2;
        }

        ControlString.getDerivedStateFromProps = function getDerivedStateFromProps(props, state) {
            if (!state.focused) {
                return { value: props.value || false };
            } else return null;
        };

        ControlString.prototype.render = function render() {
            var valueIsInvalid = this.state.dirty && this.props.validationErrors && this.props.validationErrors.length > 0;
            return _react2.default.createElement(
                _react2.default.Fragment,
                null,
                _react2.default.createElement(
                    _reactstrap.FormGroup,
                    { check: true },
                    _react2.default.createElement(
                        _reactstrap.Label,
                        { check: true },
                        _react2.default.createElement(_reactstrap.Input, {
                            type: 'checkbox',
                            checked: this.state.value,
                            disabled: !this.props.edit,
                            onChange: this.handleChange,
                            placeholder: this.props.placeholder,
                            onFocus: this.handleFocus,
                            onBlur: this.handleBlur,
                            invalid: valueIsInvalid
                        }),
                        this.props.label
                    ),
                    valueIsInvalid && renderValidationErrors(this.props)
                )
            );
        };

        return ControlString;
    }(_react.Component);
}

/**
 * @param {Type} type
 * @returns
 */
function getControlNumber(type) {
    return function (_Component3) {
        _inherits(ControlNumber, _Component3);

        function ControlNumber(props) {
            _classCallCheck(this, ControlNumber);

            var _this3 = _possibleConstructorReturn(this, _Component3.call(this, props));

            _this3.handleChange = function (value) {
                _this3.setState({ value: value, dirty: true });
                _this3.props.onChange(value);
            };

            _this3.state = { dirty: false };
            return _this3;
        }

        ControlNumber.getDerivedStateFromProps = function getDerivedStateFromProps(props) {
            return { value: props.value };
        };

        ControlNumber.prototype.render = function render() {
            var valueIsInvalid = this.state.dirty && this.props.validationErrors && this.props.validationErrors.length > 0;
            return _react2.default.createElement(
                _react2.default.Fragment,
                null,
                this.props.edit ? _react2.default.createElement(
                    _react2.default.Fragment,
                    null,
                    _react2.default.createElement(
                        _reactstrap.FormGroup,
                        null,
                        this.props.label && _react2.default.createElement(
                            _reactstrap.Label,
                            null,
                            this.props.label
                        ),
                        _react2.default.createElement(_reactNumericInput2.default, {
                            value: this.state.value,
                            onChange: this.handleChange,
                            placeholder: this.props.placeholder,
                            className: 'form-control ' + (valueIsInvalid ? 'is-invalid' : '') }),
                        valueIsInvalid && renderValidationErrors(this.props)
                    )
                ) : _react2.default.createElement(
                    _react2.default.Fragment,
                    null,
                    this.props.label && _react2.default.createElement(
                        _reactstrap.Label,
                        null,
                        this.props.label
                    ),
                    ' ',
                    this.props.value || ""
                )
            );
        };

        return ControlNumber;
    }(_react.Component);
}
/**
 * @param {Type} type
 * @returns
 */
function getControlArray(type) {

    return function (_Component4) {
        _inherits(ControlArray, _Component4);

        function ControlArray(props) {
            _classCallCheck(this, ControlArray);

            var _this4 = _possibleConstructorReturn(this, _Component4.call(this, props));

            _this4.change = function (index) {
                return function (newValue) {
                    _this4.setState({ dirty: true });
                    _this4.props.onChange(_this4.props.value.map(function (originalItem, itemIndex) {
                        return index === itemIndex ? newValue : originalItem;
                    }));
                };
            };

            _this4.add = function () {
                _this4.setState({ dirty: true });
                _this4.props.onChange([].concat(_this4.props.value || [], [null]));
            };

            _this4.remove = function (index) {
                return function () {
                    _this4.setState({ dirty: true });
                    _this4.props.onChange(_this4.props.value.filter(function (val, pos) {
                        return pos !== index;
                    }));
                };
            };

            _this4.typeControlCache = [];
            _this4.typesCache = [];

            _this4.state = { dirty: false };
            return _this4;
        }
        /**
         * @type {Array.<Type>}
         */


        ControlArray.prototype.render = function render() {
            var _this5 = this;

            //TODO : Rewrite array validation. Pass down invalid element index in validationError.
            var valueIsInvalid = this.state.dirty && this.props.validationErrors && this.props.validationErrors.length > 0;
            return _react2.default.createElement(
                _react2.default.Fragment,
                null,
                _react2.default.createElement(
                    _reactstrap.FormGroup,
                    null,
                    this.props.label && _react2.default.createElement(
                        _reactstrap.Label,
                        null,
                        this.props.label
                    ),
                    _react2.default.createElement(
                        _reactstrap.ListGroup,
                        { className: '' + (valueIsInvalid ? "border border-danger rounded" : "") },
                        this.props.value ? this.props.value.map(function (item, index) {

                            if (!_this5.typeControlCache[index]) {
                                for (var _iterator = type.getInfo("of"), _isArray = Array.isArray(_iterator), _i = 0, _iterator = _isArray ? _iterator : _iterator[Symbol.iterator]();;) {
                                    var _ref;

                                    if (_isArray) {
                                        if (_i >= _iterator.length) break;
                                        _ref = _iterator[_i++];
                                    } else {
                                        _i = _iterator.next();
                                        if (_i.done) break;
                                        _ref = _i.value;
                                    }

                                    var itemType = _ref;

                                    if (itemType.accepts(item)) {
                                        _this5.typeControlCache[index] = getControlForType(itemType);
                                        _this5.typesCache[index] = itemType;
                                        break;
                                    }
                                }
                                if (item === null || item === undefined) {
                                    // First type is default :( , this is maybe some bad design, I guess
                                    _this5.typeControlCache[index] = getControlForType(type.getInfo("of")[0]);
                                    _this5.typesCache[index] = type.getInfo("of")[0];
                                }
                                //TODO : Check before if array is valid and display error instead
                                if (!_this5.typeControlCache[index]) throw new Error("UI Lib can't render array, no type accepts " + item);
                            }

                            var TypeControl = _this5.typeControlCache[index];
                            var validationErrors = _this5.typesCache[index].getValidationErrors(item);
                            return _react2.default.createElement(
                                _reactstrap.ListGroupItem,
                                { key: index, className: 'd-flex flex-row' },
                                _react2.default.createElement(
                                    'div',
                                    { className: 'flex-fill' },
                                    _react2.default.createElement(TypeControl, {
                                        value: item,
                                        edit: _this5.props.edit,
                                        feedback: _this5.props.feedback,
                                        placeholder: _this5.props.placeholder,
                                        onChange: _this5.change(index),
                                        validationErrors: validationErrors
                                    })
                                ),
                                _this5.props.edit && _react2.default.createElement(
                                    _reactstrap.Button,
                                    {
                                        'aria-label': 'Close',
                                        className: 'align-self-start mx-2 font-weight-bold',
                                        color: 'danger',
                                        onClick: _this5.remove(index) },
                                    '\xD7'
                                )
                            );
                        }) : null,
                        this.props.edit && !(type.getInfo('array.max') <= (this.props.value || []).length) && _react2.default.createElement(
                            _reactstrap.ListGroupItem,
                            null,
                            _react2.default.createElement(
                                _reactstrap.Button,
                                {
                                    color: 'primary',
                                    'aria-label': 'Add',
                                    className: 'font-weight-bold',
                                    onClick: this.add },
                                '+'
                            )
                        )
                    ),
                    valueIsInvalid && renderValidationErrors(this.props)
                )
            );
        };

        return ControlArray;
    }(_react.Component);
}

/**
 * 
 * @param {Type} type 
 */
function getControlForType(type) {
    var control = null;

    if (!type) throw new Error("Attribute Type is " + type);

    if (type.doExtends(_types2.default.String)) control = getControlString(type);else if (type.doExtends(_types2.default.Array)) control = getControlArray(type);else if (type.doExtends(_types2.default.Number)) control = getControlNumber(type);else if (type.doExtends(_types2.default.Boolean)) control = getControlBoolean(type);else throw new Error('Type ' + type.toString() + ' not supported in bootstrap ui lib');
    return control;
}

var AttributeBase = function (_Component5) {
    _inherits(AttributeBase, _Component5);

    function AttributeBase(props) {
        _classCallCheck(this, AttributeBase);

        var _this6 = _possibleConstructorReturn(this, _Component5.call(this, props));

        _this6.onChange = function (value) {
            var _this6$props$entityCo;

            _this6.props.entityContext.merge((_this6$props$entityCo = {}, _this6$props$entityCo[_this6.props.name] = value, _this6$props$entityCo));
        };

        _this6.state = {};
        return _this6;
    }

    AttributeBase.getDerivedStateFromProps = function getDerivedStateFromProps(props, prevState) {
        if (!props.entityContext.repo.schema.typesMap[props.name]) throw new Error('Attribute ' + props.name + ' is not defined in the entity schema');

        return props.name !== prevState.name || props.entityContext.repo !== prevState.repo ? {
            control: getControlForType(props.entityContext.repo.schema.typesMap[props.name]),
            name: props.name,
            repo: props.entityContext.repo
        } : null;
    };

    AttributeBase.prototype.render = function render() {
        var Control = this.state.control;
        var eC = this.props.entityContext;
        return eC.editedValue || eC.loadedValue ? _react2.default.createElement(Control, {
            value: (eC.editedValue || eC.loadedValue)[this.props.name],
            edit: this.props.edit,
            onChange: this.onChange,
            feedback: this.props.feedback,
            placeholder: this.props.placeholder,
            label: this.props.label,
            validationErrors: eC.validationErrors ? eC.validationErrors[this.props.name] : []
        }) : null;
    };

    return AttributeBase;
}(_react.Component);

var Attribute = (0, _Entity.injectEntityContext)(AttributeBase, 'entityContext');

Attribute.propTypes = {
    name: _propTypes2.default.string.isRequired,
    edit: _propTypes2.default.bool,
    label: _propTypes2.default.node,
    placeholder: _propTypes2.default.string,
    feedback: _propTypes2.default.oneOfType([_propTypes2.default.string, _propTypes2.default.func])
};

exports.default = Attribute;
module.exports = exports['default'];