'use strict';

exports.__esModule = true;

var _extends = Object.assign || function (target) { for (var i = 1; i < arguments.length; i++) { var source = arguments[i]; for (var key in source) { if (Object.prototype.hasOwnProperty.call(source, key)) { target[key] = source[key]; } } } return target; };

var _class, _temp, _initialiseProps;

var _react = require('react');

var _react2 = _interopRequireDefault(_react);

var _propTypes = require('prop-types');

var _propTypes2 = _interopRequireDefault(_propTypes);

var _reactPromise = require('react-promise');

var _reactPromise2 = _interopRequireDefault(_reactPromise);

var _Entity = require('./Entity');

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

function _classCallCheck(instance, Constructor) { if (!(instance instanceof Constructor)) { throw new TypeError("Cannot call a class as a function"); } }

function _possibleConstructorReturn(self, call) { if (!self) { throw new ReferenceError("this hasn't been initialised - super() hasn't been called"); } return call && (typeof call === "object" || typeof call === "function") ? call : self; }

function _inherits(subClass, superClass) { if (typeof superClass !== "function" && superClass !== null) { throw new TypeError("Super expression must either be null or a function, not " + typeof superClass); } subClass.prototype = Object.create(superClass && superClass.prototype, { constructor: { value: subClass, enumerable: false, writable: true, configurable: true } }); if (superClass) Object.setPrototypeOf ? Object.setPrototypeOf(subClass, superClass) : subClass.__proto__ = superClass; }

var wrapLoadedValuesWith = function wrapLoadedValuesWith(defaultList) {
    return function (values) {
        if (!values && !defaultList) throw new Error("Entity list is " + values);
        if (!Array.isArray(values || defaultList)) throw new Error("Entity list is expected to be an array");
        return values || defaultList;
    };
};

var Entities = (_temp = _class = function (_Component) {
    _inherits(Entities, _Component);

    Entities.getInitialState = function getInitialState(_ref) {
        var repo = _ref.repo,
            entitiesRef = _ref.entitiesRef,
            defaultList = _ref.defaultList,
            sync = _ref.sync;

        return {
            entitiesRef: entitiesRef, //Just to compare on props change
            repo: repo, // Just to compare on props change
            defaultList: defaultList, // Just to compare on props change
            sync: sync // Just to compare on props change
        };
    };

    Entities.getDerivedStateFromProps = function getDerivedStateFromProps(props, state) {
        if (props.entitiesRef !== state.entitiesRef || props.repo !== state.repo || props.sync !== state.sync //TODO: We can have performance optimization here by : when true -> false : disable sync, false -> true : start it
        || !props.entitiesRef && props.defaultList !== state.defaultList) {
            return Entities.getInitialState(props);
        } else return null;
    };

    function Entities(props) {
        _classCallCheck(this, Entities);

        var _this = _possibleConstructorReturn(this, _Component.call(this, props));

        _initialiseProps.call(_this);

        _this.state = {};
        return _this;
    }

    Entities.prototype._loadData = function _loadData() {
        var _this2 = this;

        var _props = this.props,
            repo = _props.repo,
            entitiesRef = _props.entitiesRef,
            defaultList = _props.defaultList;

        if (this._loadingPromise) {
            //TODO: Cancel here when repo request will be cancellable
            this._loadingPromise = null;
        }
        var loadingPromise = entitiesRef ? repo.read(entitiesRef).then(wrapLoadedValuesWith(defaultList)) : defaultList ? Promise.resolve(defaultList) : Promise.reject(new Error("You should define at least an entitiesRef or a defaultList on <Entities/>"));
        this._loadingPromise = loadingPromise;

        this.setState(function (prevState) {
            return { loadingPromise: loadingPromise };
        }, function () {
            loadingPromise.then(function (loadedValues) {
                if (_this2._loadingPromise === loadingPromise) {
                    _this2.setState({ loadedValues: loadedValues });
                    if (_this2.props.sync && entitiesRef) {
                        _this2._startWatch();
                    }
                    _this2._loadingPromise = null;
                }
            }).catch(console.warn); // Just log here cause, Async will pass it to user.
        });
    };

    Entities.prototype._startWatch = function _startWatch() {
        var _this3 = this;

        var _props2 = this.props,
            repo = _props2.repo,
            entitiesRef = _props2.entitiesRef,
            defaultList = _props2.defaultList;

        this._stopWatch = repo.watch(entitiesRef, function (changedValues) {
            return _this3.setState({
                loadedValues: wrapLoadedValuesWith(defaultList)(changedValues),
                syncTimestamp: new Date() });
        }, function (error) {
            return _this3.setState({ syncError: error });
        });
    };

    Entities.prototype.componentDidMount = function componentDidMount() {
        this._loadData();
    };

    Entities.prototype.componentDidUpdate = function componentDidUpdate() {
        if (this.state.loadingPromise === null) {
            this._loadData();
        }
    };

    Entities.prototype.componentWillUnmount = function componentWillUnmount() {
        if (this._loadingPromise) {
            //TODO: Cancel here when repo request will be cancellable
            this._loadingPromise = null;
        }
        if (this._stopWatch) {
            this._stopWatch();
            this._stopWatch = null;
        }
    };

    Entities.prototype.render = function render() {
        var _this4 = this;

        return this.state.loadingPromise ? _react2.default.createElement(_reactPromise2.default, {
            promise: this.state.loadingPromise,
            then: this.renderPostLoading,
            'catch': function _catch(error) {
                return _this4.renderChildren({ isLoading: false, loadingError: error });
            },
            pending: function pending() {
                return _this4.renderChildren({ isLoading: true });
            }
        }) : this.renderChildren({ isLoading: true });
    };

    return Entities;
}(_react.Component), _initialiseProps = function _initialiseProps() {
    var _this5 = this;

    this.commands = {
        reload: function reload() {
            _this5.setState(function (prevState, props) {
                return Entities.getInitialState(props);
            });
        }
    };

    this.renderChildren = function (_ref2) {
        var isLoading = _ref2.isLoading,
            loadingError = _ref2.loadingError,
            loadedValues = _ref2.loadedValues;

        return _react2.default.createElement(
            _Entity.EntityContext.Provider,
            { value: _extends({
                    repo: _this5.props.repo,
                    entitiesRef: _this5.props.entitiesRef,
                    defaultList: _this5.props.defaultList,
                    // Loading
                    loadedValues: _this5.state.loadedValues || loadedValues,
                    loadingPromise: _this5.state.loadingPromise,
                    isLoading: !!isLoading,
                    loadingError: loadingError,
                    // Sync
                    syncTimestamp: _this5.state.syncTimestamp,
                    syncError: _this5.state.syncError
                }, _this5.commands) },
            typeof _this5.props.children === 'function' ? _this5.props.children(_this5.state.loadedValues || loadedValues, {
                isLoading: !!isLoading, loadingError: loadingError,
                loadedValues: _this5.state.loadedValues || loadedValues
            }, _this5.commands) : _this5.props.children
        );
    };

    this.renderPostLoading = function (loadedValues) {
        return _this5.renderChildren({ loadedValues: loadedValues });
    };
}, _temp);
//TODO: Document all proptypes

Entities.propTypes = {
    sync: _propTypes2.default.bool
};

exports.default = Entities;
module.exports = exports['default'];