'use strict';

exports.__esModule = true;
exports.injectEntityContext = exports.EntityContext = undefined;

var _class, _temp, _initialiseProps;

var _extends = Object.assign || function (target) { for (var i = 1; i < arguments.length; i++) { var source = arguments[i]; for (var key in source) { if (Object.prototype.hasOwnProperty.call(source, key)) { target[key] = source[key]; } } } return target; };

var _react = require('react');

var _react2 = _interopRequireDefault(_react);

var _propTypes = require('prop-types');

var _propTypes2 = _interopRequireDefault(_propTypes);

var _EntityRepo = require('../../model/core/EntityRepo');

var _EntityRepo2 = _interopRequireDefault(_EntityRepo);

var _reactPromise = require('react-promise');

var _reactPromise2 = _interopRequireDefault(_reactPromise);

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

function _classCallCheck(instance, Constructor) { if (!(instance instanceof Constructor)) { throw new TypeError("Cannot call a class as a function"); } }

function _possibleConstructorReturn(self, call) { if (!self) { throw new ReferenceError("this hasn't been initialised - super() hasn't been called"); } return call && (typeof call === "object" || typeof call === "function") ? call : self; }

function _inherits(subClass, superClass) { if (typeof superClass !== "function" && superClass !== null) { throw new TypeError("Super expression must either be null or a function, not " + typeof superClass); } subClass.prototype = Object.create(superClass && superClass.prototype, { constructor: { value: subClass, enumerable: false, writable: true, configurable: true } }); if (superClass) Object.setPrototypeOf ? Object.setPrototypeOf(subClass, superClass) : subClass.__proto__ = superClass; }

var EntityContext = exports.EntityContext = _react2.default.createContext();
/**
 * 
 * @param {Component} Component 
 * @param {string} [propName = entityContext]
 */
var injectEntityContext = exports.injectEntityContext = function injectEntityContext(Component, propName) {
    return function (props) {
        return _react2.default.createElement(
            EntityContext.Consumer,
            null,
            function (entityContext) {
                var _ref;

                return _react2.default.createElement(Component, _extends({}, props, (_ref = {}, _ref[propName || 'entityContext'] = entityContext, _ref)));
            }
        );
    };
};

var wrapLoadedValueWith = function wrapLoadedValueWith(defaultValue) {
    return function (value) {
        if (Array.isArray(value)) console.warn("The value loaded in <Entity/> is an array, this is not supported by <Entity/>. You should use <Entities/> instead.");
        if (!value && !defaultValue) throw new Error("Entity doesn't exist");
        return _extends({}, defaultValue || {}, value || {});
    };
};

var initialValidationInfo = {
    validationPromise: null,
    isValid: undefined,
    isValidating: true,
    validationErrorsCount: undefined,
    validationErrors: undefined
};

var initialSavingInfo = {
    savingPromise: null,
    isSaving: false,
    savingError: undefined,
    isSaved: false
};

var initialSyncInfo = {
    syncTimestamp: undefined,
    syncError: undefined
};
var Entity = (_temp = _class = function (_Component) {
    _inherits(Entity, _Component);

    Entity.getInitialState = function getInitialState(_ref2) {
        var repo = _ref2.repo,
            entityRef = _ref2.entityRef,
            defaultValue = _ref2.defaultValue,
            sync = _ref2.sync;

        return _extends({
            entityRef: entityRef, //Just to compare on props change
            repo: repo, // Just to compare on props change
            defaultValue: defaultValue, // Just to compare on props change
            sync: sync, // Just to compare on props change

            editedValue: null,

            loadedValue: null,
            loadingPromise: null

        }, initialValidationInfo, initialSavingInfo, initialSyncInfo);
    };

    Entity.getDerivedStateFromProps = function getDerivedStateFromProps(props, state) {
        if (props.entityRef !== state.entityRef || props.repo !== state.repo || props.sync !== state.sync //TODO: We can have performance optimization here : when true -> false : disable sync, false -> true : start it
        || !props.entityRef && props.defaultValue !== state.defaultValue) {
            if (state.editedValue) console.warn("Resetting state on an entity view with pending modification. Make sure this is really what you want to do.");
            return Entity.getInitialState(props);
        } else return null;
    };

    function Entity(props) {
        _classCallCheck(this, Entity);

        var _this = _possibleConstructorReturn(this, _Component.call(this, props));

        _initialiseProps.call(_this);

        _this.state = {};
        return _this;
    }

    Entity.prototype._loadData = function _loadData() {
        var _this2 = this;

        var _props = this.props,
            repo = _props.repo,
            entityRef = _props.entityRef,
            defaultValue = _props.defaultValue;


        if (this._loadingPromise) {
            //TODO: Cancel here when repo request will be cancellable
            this._loadingPromise = null;
        }
        var loadingPromise = entityRef ? repo.read(entityRef).then(wrapLoadedValueWith(defaultValue)) : defaultValue ? Promise.resolve(defaultValue) : Promise.reject(new Error("You should define at least an entityRef or a defaultValue on <Entity/>"));
        this._loadingPromise = loadingPromise;

        this.setState(function (prevState) {
            return { loadingPromise: loadingPromise };
        }, function () {
            loadingPromise.then(function (loadedValue) {
                if (_this2._loadingPromise === loadingPromise) {
                    _this2.setState({ loadedValue: loadedValue });
                    if (_this2.props.sync && entityRef) {
                        _this2._startWatch();
                    }
                    _this2._loadingPromise = null;
                }
            }).catch(console.warn); // Just log here cause, Async will pass it to user.
        });
    };

    Entity.prototype._startWatch = function _startWatch() {
        var _this3 = this;

        var _props2 = this.props,
            repo = _props2.repo,
            entityRef = _props2.entityRef,
            defaultValue = _props2.defaultValue;

        this._stopWatch = repo.watch(entityRef, function (changedValue) {
            return _this3.setState({
                loadedValue: wrapLoadedValueWith(defaultValue)(changedValue),
                syncTimestamp: new Date() });
        }, function (error) {
            return _this3.setState({ syncError: error });
        });
    };

    Entity.prototype._validateData = function _validateData() {
        var _this4 = this;

        var repo = this.props.repo;

        var value = this.state.editedValue || this.state.loadedValue;
        if (this._validationPromise) {
            //TODO: Cancel here (especially if we use backend validation)
            this._validationPromise = null;
        }
        var validationPromise = repo.validate(value);
        this._validationPromise = validationPromise;
        this.setState(function (prevState) {
            return { validationPromise: validationPromise, isValidating: true };
        }, function () {
            validationPromise.then(function (validationErrors) {
                if (_this4._validationPromise === validationPromise) {
                    var validationErrorsCount = 0;
                    for (var attribute in validationErrors) {
                        validationErrorsCount += validationErrors[attribute].length;
                    }
                    _this4.setState({
                        isValid: validationErrorsCount === 0,
                        validationErrorsCount: validationErrorsCount,
                        isValidating: false,
                        validationErrors: validationErrors
                    });
                    _this4._validationPromise = null;
                }
            });
        });
    };

    Entity.prototype.componentDidMount = function componentDidMount() {
        this._loadData();
    };

    Entity.prototype.componentDidUpdate = function componentDidUpdate() {
        if (this.state.loadingPromise === null) {
            this._loadData();
        } else if (this.state.loadedValue !== null && this.state.validationPromise === null) {
            this._validateData();
        }
    };

    Entity.prototype.componentWillUnmount = function componentWillUnmount() {
        if (this._loadingPromise) {
            //TODO: Cancel here when repo request will be cancellable
            this._loadingPromise = null;
        }
        if (this._validationPromise) {
            //TODO: Cancel here (especially if we use backend validation)
            this._validationPromise = null;
        }
        if (this._stopWatch) {
            this._stopWatch();
            this._stopWatch = null;
        }
    };

    Entity.prototype.render = function render() {
        var _this5 = this;

        return this.state.loadingPromise ? _react2.default.createElement(_reactPromise2.default, {
            promise: this.state.loadingPromise,
            then: this.renderPostLoading,
            'catch': function _catch(error) {
                return _this5.renderChildren({ isLoading: false, loadingError: error });
            },
            pending: function pending() {
                return _this5.renderChildren({ isLoading: true });
            }
        }) : this.renderChildren({ isLoading: true });
    };

    return Entity;
}(_react.Component), _initialiseProps = function _initialiseProps() {
    var _this6 = this;

    this.commands = {
        reload: function reload() {
            _this6.setState(function (prevState, props) {
                return Entity.getInitialState(props);
            });
        },
        reset: function reset() {
            _this6.setState(_extends({ editedValue: null }, initialValidationInfo));
        },
        merge: function merge(newValue) {
            _this6.setState(function (prevState) {
                return _extends({
                    editedValue: _extends({}, prevState.editedValue || prevState.loadedValue, newValue)
                }, initialValidationInfo, {
                    isSaved: false
                });
            });
        },
        set: function set(newValue) {
            _this6.setState(_extends({
                editedValue: newValue
            }, initialValidationInfo, {
                isSaved: false
            }));
        },
        save: function save() {
            if (!_this6.state.editedValue) {
                console.warn("You tried to save although no edit have been made to the value. It is a noop, but it means something is wrong");
            } else {
                var savingPromise = _this6.props.repo.save(_this6.state.editedValue, _this6.props.entityRef);
                _this6.setState(function (prevState) {
                    return { savingPromise: savingPromise, isSaving: true };
                }, function () {
                    savingPromise.then(function (ref) {
                        _this6.setState(_extends({}, initialSavingInfo, { isSaved: true }));
                        if (typeof _this6.props.onSave === 'function') _this6.props.onSave(ref);
                    }).catch(function (error) {
                        _this6.setState(_extends({}, initialSavingInfo, { savingError: error }));
                    });
                });
            }
        }
    };

    this.renderChildren = function (_ref3) {
        var isLoading = _ref3.isLoading,
            loadingError = _ref3.loadingError,
            loadedValue = _ref3.loadedValue;

        return _react2.default.createElement(
            EntityContext.Provider,
            { value: _extends({
                    repo: _this6.props.repo,
                    entityRef: _this6.props.entityRef,
                    defaultValue: _this6.props.defaultValue,
                    // Loading
                    loadedValue: _this6.state.loadedValue || loadedValue,
                    loadingPromise: _this6.state.loadingPromise,
                    editedValue: _this6.state.editedValue,
                    isLoading: !!isLoading,
                    loadingError: loadingError,
                    // Sync
                    syncTimestamp: _this6.state.syncTimestamp,
                    syncError: _this6.state.syncError,
                    // Validation
                    isValid: _this6.state.isValid,
                    validationErrors: _this6.state.validationErrors,
                    isValidating: _this6.state.isValidating,
                    validationErrorsCount: _this6.state.validationErrorsCount,
                    // Saving
                    savingPromise: _this6.state.savingPromise,
                    isSaving: _this6.state.isSaving,
                    savingError: _this6.state.savingError,
                    isSaved: _this6.state.isSaved
                }, _this6.commands) },
            typeof _this6.props.children === 'function' ? _this6.props.children(_this6.state.editedValue || _this6.state.loadedValue || loadedValue, {
                isLoading: !!isLoading,
                loadingError: loadingError,
                loadedValue: _this6.state.loadedValue || loadedValue,
                isDirty: !!_this6.state.editedValue,
                // Validation
                isValid: _this6.state.isValid, validationErrors: _this6.state.validationErrors,
                isValidating: _this6.state.isValidating, validationErrorsCount: _this6.state.validationErrorsCount,
                // Saving
                isSaving: _this6.state.isSaving,
                savingError: _this6.state.savingError,
                isSaved: _this6.state.isSaved

            }, _this6.commands) : _this6.props.children
        );
    };

    this.renderPostLoading = function (loadedValue) {
        return _this6.renderChildren({ loadedValue: loadedValue });
    };
}, _temp);


Entity.propTypes = {
    /**
     * The EntityRepo where this entity belongs.
    */
    repo: _propTypes2.default.instanceOf(_EntityRepo2.default).isRequired,
    /** 
     * The reference of the entity to display or edit. EntityRef are
     * specific to persistence provider.
    */
    entityRef: _propTypes2.default.any,
    /** 
     * This value will be used if entityRef is not set or if it load 
     * an empty, or null or undefined value from the backend.
    */
    defaultValue: _propTypes2.default.object,
    /**
     * Entity can take either anything that can be rendered or a function.
     * 
     * When it is a function, the function will take three parameters, the value loaded,
     * an object with information (isLoading, isValid...), 
     * and an object with commands (set, save...).
     * The function should return anything that can be rendered.
     * 
     * Entity also provide an EntityContext that is used by extensions.
    */
    children: _propTypes2.default.oneOfType([_propTypes2.default.func, _propTypes2.default.node]),
    /** 
     * Callback called each time the value is saved.
    */
    onSave: _propTypes2.default.func
};

exports.default = Entity;